#include "../../libs/communication/communication.h"
#include "../../libs/protocol/protocol.h"
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>

void create_directory(const char *path, mode_t mode) {
	struct stat st = {0};
	if (stat(path, &st) == -1) {
		if (mkdir(path, mode) == -1) {
			fprintf(stderr, "Error creating directory %s: %s\n", path, strerror(errno));
			exit(EXIT_FAILURE);
		}
	}
}

void write_message_to_file(const char *directory, const char *filename, mode_t mode,
						   unsigned char *msg) {
	char filepath[PATH_MAX];
	snprintf(filepath, sizeof(filepath), "%s/%s", directory, filename);
	int fd = open(filepath, O_WRONLY | O_CREAT | O_EXCL, mode);
	if (fd == -1) {
		fprintf(stderr, "Error creating file %s: %s\n", filepath, strerror(errno));
		exit(EXIT_FAILURE);
	}
	if (write(fd, msg, MESSAGE_SIZE) == -1) {
		fprintf(stderr, "Error writing to file %s: %s\n", filepath, strerror(errno));
		exit(EXIT_FAILURE);
	}
	close(fd);
}

int main() {

	unsigned char buffer[MESSAGE_SIZE];
	read(0, buffer, MESSAGE_SIZE);
	message msg;
	deserialize_message(buffer, MESSAGE_SIZE, &msg);

	if (msg.header.isgroup) {
		const char *directory_format = "/opt/djumbai/group/%d/message_box";
		char directory[PATH_MAX];
		snprintf(directory, sizeof(directory), directory_format, msg.header.receiver);

		// Ensure that the directory structure exists
		create_directory(directory, 0070);

		// Write the message to a file inside the sender's folder
		char timestamp_str[21];
		unsigned long long timestamp =
			msg.header.timestamp.tv_sec * 1000 + msg.header.timestamp.tv_usec / 1000;
		snprintf(timestamp_str, sizeof(timestamp_str), "%lld", timestamp);

		write_message_to_file(directory, timestamp_str, 0020, buffer);

	} else {
		const char *directory_format = "/opt/djumbai/user/%d/message_box/%d";
		char directory[PATH_MAX];
		snprintf(directory, sizeof(directory), directory_format, msg.header.receiver,
				 msg.header.sender);

		// Ensure that the directory structure exists
		create_directory(directory, 0700);

		// Write the message to a file inside the sender's folder
		char timestamp_str[21];
		unsigned long long timestamp =
			msg.header.timestamp.tv_sec * 1000 + msg.header.timestamp.tv_usec / 1000;
		snprintf(timestamp_str, sizeof(timestamp_str), "%lld", timestamp);

		write_message_to_file(directory, timestamp_str, 0200, buffer);
	}
}
